﻿import com.robertpenner.profmx.snow.Snowstorm;

/**
* The Snow class.
* 
* @author Robert Penner (AS 1.0)
* © 2002 Robert Penner
* http://www.robertpenner.com
* 
* @author Mark Walters (AS 2.0)
* © 2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 6
*/
class com.robertpenner.profmx.snow.Snow extends MovieClip {
	
	/*
	* ON STAGE
	*/
	private var boundingBox:MovieClip;
	
	/*
	* CLASS PROPERTIES
	*/
	private var $numFlakes:Number = 0;
	private var $storm:Snowstorm;
	
	/*
	* CLIP PARAMETERS
	*/
	private var $clipParameters:Object = {maxFlakes: 1, fallSpeed: 1, friction: 1, zDepth: 1};
	
	/*
	* INSPECTABLE PROPERTIES
	*/
	private var $maxFlakes:Number;
	private var $fallSpeed:Number;
	private var $friction:Number;
	private var $zDepth:Number;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	
	function Snow () {
		this.initFromClipParameters();
	}
	
	
	/*
	* *********************************************************
	* PRIVATE METHODS
	* *********************************************************
	* 
	*/
	
	private function initFromClipParameters ():Void {
		var key:String, found:Boolean = false;
		for(key in $clipParameters) {
			if(this.hasOwnProperty(key)) {
				found = true;
				this["def_" + key] = this[key];
				delete this[key];
			}
		}
		if(found) {
			for(key in $clipParameters) {
				var v = this["def_" + key];
				if(v != undefined) {
					this[key] = v;
				}
			}
		}
	}
	
	private function init ():Void {
		var bleed:Number = 10;
		var limits:Object = {
			xmin: 0 - bleed,
			xmax: this._width + bleed,
			ymin: 0 - bleed,
			ymax: this._height + bleed,
			zmin: 0,
			zmax: this.zDepth
		};
		this.$storm = new Snowstorm (limits,
									this.friction,
									this.fallSpeed, 
									this);
		this._xscale = this._yscale = 100;
		this.boundingBox._visible = false;
		this.boundingBox._width = 0;
		this.boundingBox._height = 0;
		this.onEnterFrame = this.makeFlake;
	}
	
	private function makeFlake ():Void {
		if (this.$numFlakes++ < this.maxFlakes) {
			this.$storm.addFlake();
		} else {
			delete this.onEnterFrame;
		}
	}
	
	
	/*
	* *********************************************************
	* EVENT HANDLERS
	* *********************************************************
	* 
	*/
	
	private function onMouseMove ():Void {
		var half:Number = this._width/2;
		this.$storm.wind = (this._xmouse - half) / half;
	}
	
	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The maxFlakes property of the Snow.
	* 
	* @usage
	* <pre><code>
	* trace (snow.maxFlakes);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=80)]
	public function get maxFlakes ():Number {
		return this.$maxFlakes;
	}
	public function set maxFlakes (mf:Number):Void {
		 this.$maxFlakes = mf;
	}
	
	/**
	* The fallSpeed property of the Snow.
	* 
	* @usage
	* <pre><code>
	* trace (snow.fallSpeed);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=5)]
	public function get fallSpeed ():Number {
		return this.$fallSpeed;
	}
	public function set fallSpeed (fs:Number):Void {
		 this.$fallSpeed = fs;
	}
	
	/**
	* The friction property of the Snow.
	* 
	* @usage
	* <pre><code>
	* trace (snow.friction);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=.03)]
	public function get friction ():Number {
		return this.$friction;
	}
	public function set friction (f:Number):Void {
		 this.$friction = f;
	}
	
	/**
	* The zDepth property of the Snow.
	* 
	* @usage
	* <pre><code>
	* trace (snow.zDepth);
	* </code></pre>
	*/
	[Inspectable(type=Number, defaultValue=800)]
	public function get zDepth ():Number {
		return this.$zDepth;
	}
	public function set zDepth (zd:Number):Void {
		 this.$zDepth = zd;
	}
	
	/**
	* The storm property of the Snow.
	* 
	* @usage
	* <pre><code>
	* trace (snow.storm);
	* </code></pre>
	*/
	public function get storm ():Snowstorm {
		return this.$storm;
	}
	
}
