﻿import mx.transitions.OnEnterFrameBeacon;
import com.robertpenner.geom.Vector3d;
import com.robertpenner.utils.MathUtil;
import com.robertpenner.profmx.snow.Snowstorm;

/**
* The Snowflake class encapsulates functionality for a falling snow animation.
* Designed to collaborate with the Snowstorm class.
* 
* @author Robert Penner (AS 1.0)
* © 2002 Robert Penner
* http://www.robertpenner.com
* 
* @author Mark Walters (AS 2.0)
* © 2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 6
*/
class com.robertpenner.profmx.snow.Snowflake {
	
	/*
	* CLASS PROPERTIES
	*/
	private var $timeline:MovieClip;
	private var $mc:MovieClip;
	private var $pos:Vector3d;
	private var $vel:Vector3d;
	private var $storm:Snowstorm;
	private var $velRot:Number = 0;
	private var $rotation:Number = 0;
	private var $t:Number = 0;
	private var $brown:Number = 0;
	private var $pers:Number = 0;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	
	/**
	* Creates an instance of the Snowflake class.
	* 
	* @param	fallSpeed	The velocity of the Snowflake.
	* @param	timeline	A reference to the MovieClip that contains this particular Snowflake.
	* @param	depth		An integer specifying the depth level where the Snowflake is placed.
	* 
	* @usage
	* <pre><code>
	* var flake:Snowflake = new Snowflake(5, this, 1);
	* </code></pre>
	*/
	function Snowflake (fallSpeed:Number, timeline:MovieClip, depth:Number) {
		this.$timeline = timeline;
		this.$mc = this.$timeline.createEmptyMovieClip ("s" + depth, depth);
		com.robertpenner.display.ColorClip.mixin(this.$mc);
		com.robertpenner.display.DrawingClip.mixin(this.$mc);
		this.draw();
		this.$pos = new Vector3d(0, 0, 0);
		this.$vel = new Vector3d(0, -fallSpeed, 0);
		this.$velRot = MathUtil.randRangeFloat (-10, 10);
		OnEnterFrameBeacon.init();
		MovieClip.addListener(this);
	}
	
	
	/*
	* *********************************************************
	* PRIVATE METHODS
	* *********************************************************
	* 
	*/
	
	private function draw ():Void {
		with (this.$mc) {
			beginFill(0xFFFFFF);
			drawRegPoly(0, 0, 10, 6);
			endFill();
		}
	}
	
	private function move ():Void {
		if (++this.$t % 10 == 0) {
			this.$brown = MathUtil.randRangeFloat (-.2, .2);
		}
		this.$vel.x += this.$brown + this.$storm.wind;
		this.$vel.x *= 1 - this.$storm.friction;
		this.$pos.plus (this.$vel);
		this.$rotation += this.$velRot;
		this.renderXY();
	}
	
	private function onEnterFrame ():Void {
		this.move();
	}
	
	private function renderXY ():Void {
		this.$mc._x = this.$pos.x * this.$pers;
		this.$mc._y = -this.$pos.y * this.$pers;
		this.$mc._rotation = this.$rotation;
		if (this.$mc._y > this.$storm.limits.ymax) this.screenY = this.$storm.limits.ymin;
		if (this.$mc._x > this.$storm.limits.xmax) this.screenX = this.$storm.limits.xmin
		else if (this.$mc._x < this.$storm.limits.xmin) this.screenX = this.$storm.limits.xmax;
	}
	
	private function renderZ ():Void {
		this.$pers = this.$pos.getPerspective();
		this.$mc.swapDepths (Math.floor (100000 - this.$pos.z * 100));
		this.$mc._xscale = this.$mc._yscale = 100 * this.$pers;
		this.$mc.setBrightness (-100 * (1 - this.$pers));
	}
	
	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The x and y properties of the Snowflake.
	* 
	* @usage
	* <pre><code>
	* flake.xy = {x:10, y:10};
	* </code></pre>
	*/
	public function set xy (xy:Object):Void {
		this.$pos.x = xy.x;
		this.$pos.y = xy.y;
		this.renderXY();
	}
	
	/**
	* The z property of the Snowflake.
	* 
	* @usage
	* <pre><code>
	* flake.z = 10;
	* </code></pre>
	*/
	public function set z (z:Number):Void {
		this.$pos.z = z;
		this.renderZ();
	}
	
	/**
	* The screenX property of the Snowflake.
	* 
	* @usage
	* <pre><code>
	* flake.screenX = 10;
	* </code></pre>
	*/
	public function set screenX (x:Number):Void {
		this.$pos.x = (this.$mc._x = x) / this.$pers;
	}
	
	/**
	* The screenY property of the Snowflake.
	* 
	* @usage
	* <pre><code>
	* flake.screenY = 10;
	* </code></pre>
	*/
	public function set screenY (y:Number):Void {
		this.$pos.y = -(this.$mc._y = y) / this.$pers;
	}
	
	/**
	* The screenXY property of the Snowflake.
	* 
	* @usage
	* <pre><code>
	* flake.screenXY = {x:10, y:10};
	* </code></pre>
	*/
	public function set screenXY (xy:Object):Void {
		this.screenX = xy.x;
		this.screenY = xy.y;
	}
	
	/**
	* The storm property of the Snowflake.
	* 
	* @usage
	* <pre><code>
	* trace (flake.storm);
	* </code></pre>
	*/
	public function get storm ():Snowstorm {
		return this.$storm;
	}
	public function set storm (s:Snowstorm):Void {
		this.$storm = s;
	}
	
}
