﻿import com.robertpenner.utils.MathUtil;
import com.robertpenner.profmx.snow.Snowflake;

/**
* The Snowstorm class manages a collection of Snowflake objects,
* defining their limits, friction, falling speed, and wind.
* 
* @author Robert Penner (AS 1.0)
* © 2002 Robert Penner
* http://www.robertpenner.com
* 
* @author Mark Walters (AS 2.0)
* © 2006 DigitalFlipbook
* http://www.digitalflipbook.com
* 
* @version	1.0
* 
* @since	ActionScript 2.0; Flash Player 6
*/
class com.robertpenner.profmx.snow.Snowstorm {
	
	/*
	* CLASS PROPERTIES
	*/
	private var $limits:Object;
	private var $friction:Number = 0;
	private var $fallSpeed:Number = 0;
	private var $timeline:MovieClip;
	private var $flakes:Array;
	private var $wind:Number = 0;
	
	
	/*
	* *********************************************************
	* CONSTRUCTOR
	* *********************************************************
	*
	*/
	
	/**
	* Creates an instance of the Snowstorm class.
	* 
	* @param	fallSpeed	The velocity of the Snowflake.
	* @param	timeline	A reference to the MovieClip that contains this particular Snowflake.
	* @param	depth		An integer specifying the depth level where the Snowflake is placed.
	* 
	* @usage
	* <pre><code>
	* var limits:Object = {xmin:0, xmax:this._width, ymin:0, ymax:this._height, zmin:0, zmax:800};
	* var storm:Snowstorm = new Snowstorm(limits, .03, 5, this);
	* </code></pre>
	*/
	function Snowstorm (limits:Object, friction:Number, fallSpeed:Number, timeline:MovieClip) {
		this.$limits = limits;
		this.$friction = friction;
		this.$fallSpeed = fallSpeed;
		this.$timeline = timeline;
		this.$flakes = [];
	}
	
	
	/*
	* *********************************************************
	* PUBLIC METHODS
	* *********************************************************
	* 
	*/
	
	/**
	* Adds a snowflake to a Snowstorm instance.
	* 
	* @usage
	* <pre><code>
	* storm.addFlake();
	* </code></pre>
	*/
	public function addFlake ():Void {
		var newFlake:Snowflake = new Snowflake (this.$fallSpeed, this.$timeline, this.$flakes.length);
		newFlake.storm = this;
		var x:Number = MathUtil.randRangeFloat (this.$limits.xmin, this.$limits.xmax);
		var y:Number = this.$limits.ymin;
		var z:Number = MathUtil.randRangeFloat (this.$limits.zmin, this.$limits.zmax);
		newFlake.z = z;
		newFlake.screenXY = {x:x, y:y};
		this.$flakes.unshift (newFlake);
	}
	
	/**
	* Removes a snowflake from a Snowstorm instance.
	* 
	* @usage
	* <pre><code>
	* storm.removeFlake();
	* </code></pre>
	*/
	public function removeFlake ():Object {
		return this.$flakes.pop();
	}
	
	
	/*
	* *********************************************************
	* GETTERS/SETTERS
	* *********************************************************
	* 
	*/
	
	/**
	* The limits property of the Snowstorm.
	* 
	* @usage
	* <pre><code>
	* trace (storm.limits.xmin);
	* trace (storm.limits.xmax);
	* trace (storm.limits.ymin);
	* trace (storm.limits.ymax);
	* trace (storm.limits.zmin);
	* trace (storm.limits.zmax);
	* </code></pre>
	*/
	public function get limits ():Object {
		return this.$limits;
	}
	public function set limits (l:Object):Void {
		this.$limits = l;
	}
	
	/**
	* The friction property of the Snowstorm.
	* 
	* @usage
	* <pre><code>
	* trace (storm.friction);
	* </code></pre>
	*/
	public function get friction ():Number {
		return this.$friction;
	}
	public function set friction (f:Number):Void {
		this.$friction = f;
	}
	
	/**
	* The fallSpeed property of the Snowstorm.
	* 
	* @usage
	* <pre><code>
	* trace (storm.fallSpeed);
	* </code></pre>
	*/
	public function get fallSpeed ():Number {
		return this.$fallSpeed;
	}
	public function set fallSpeed (s:Number):Void {
		this.$fallSpeed = s;
	}
	
	/**
	* The flakes property of the Snowstorm.
	* 
	* @usage
	* <pre><code>
	* trace (storm.flakes.length);
	* </code></pre>
	*/
	public function get flakes ():Array {
		return this.$flakes;
	}
	
	/**
	* The wind property of the Snowstorm.
	* 
	* @usage
	* <pre><code>
	* trace (storm.wind);
	* </code></pre>
	*/
	public function get wind ():Number {
		return this.$wind;
	}
	public function set wind (w:Number):Void {
		this.$wind = w;
	}
	
}
